package co.hyperverge.hypersnapsdk.hypersnapsdk_flutter;

import android.content.Context;
import android.util.Log;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import co.hyperverge.hypersnapsdk.activities.HVDocsActivity;
import co.hyperverge.hypersnapsdk.listeners.DocCaptureCompletionHandler;
import co.hyperverge.hypersnapsdk.objects.HVDocConfig;
import co.hyperverge.hypersnapsdk.objects.HVError;
import co.hyperverge.hypersnapsdk.objects.HVResponse;


public class FLHVDocsCapture {

    private static final String TAG = FLHVDocsCapture.class.getSimpleName();

    private HVDocConfig docConfig;
    private Float aspectRatio;
    private boolean hasBeenCalled;

    public void createNewDocConfig() {
        this.docConfig = new HVDocConfig();
    }

    public HVDocConfig getDocConfig() {
        if (this.docConfig == null) {
            this.docConfig = new HVDocConfig();
        }
        return this.docConfig;
    }

    @SuppressWarnings("unchecked")
    public void setHVDocConfig(Map<String, Object> hvDocConfig) {
        Double aspectRatioValue = (Double) hvDocConfig.get(FLHyperSnapConstants.DOC_ASPECT_RATIO_VALUE);
        Boolean shouldSetPadding = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_SHOULD_SET_PADDING);
        Double padding = (Double) hvDocConfig.get(FLHyperSnapConstants.DOC_DEFAULT_PADDING_VALUE);
        Boolean shouldShowFlashIcon = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_SHOULD_SHOW_FLASH_ICON);
        Boolean shouldShowReviewScreen = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_SHOULD_SHOW_REVIEW_SCREEN);
        Boolean shouldShowInstructionPage = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_SHOULD_SHOW_INSTRUCTION_PAGE);
        Boolean shouldEnableRetries = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_SHOULD_ENABLE_RETRIES);
        String docTypeValue = (String) hvDocConfig.get(FLHyperSnapConstants.DOC_TYPE_VALUE);
        Boolean shouldShowTrustLogos = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_SHOULD_SHOW_TRUST_LOGOS);
        Boolean shouldShowCloseAlert = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_SHOULD_SHOW_CLOSE_ALERT);
        String closeAlertDialogTitle = (String) hvDocConfig.get(FLHyperSnapConstants.DOC_CLOSE_ALERT_DIALOG_TITLE);
        String closeAlertDialogDescription = (String) hvDocConfig.get(FLHyperSnapConstants.DOC_CLOSE_ALERT_DIALOG_DESCRIPTION);
        Integer autoCaptureDuration = (Integer) hvDocConfig.get(FLHyperSnapConstants.DOC_AUTO_CAPTURE_DURATION);
        Boolean shouldDocAutoCapture = (Boolean) hvDocConfig.get(FLHyperSnapConstants.DOC_AUTO_CAPTURE);
        Map<String, Object> ocrApiDetails = (Map<String, Object>) hvDocConfig.get(FLHyperSnapConstants.OCR_API_DETAILS);
        Map<String, Object> docCaptureUIStrings = (Map<String, Object>) hvDocConfig.get(FLHyperSnapConstants.DOC_CAPTURE_UI_STRINGS);

        if (aspectRatioValue != null) {
            aspectRatio = aspectRatioValue.floatValue();
        }
        if (shouldSetPadding != null) {
            getDocConfig().setShouldAddPadding(shouldSetPadding);
        }
        if (padding != null) {
            getDocConfig().setPadding(padding.floatValue());
        }
        if (shouldShowFlashIcon != null) {
            getDocConfig().setShouldShowFlashIcon(shouldShowFlashIcon);
        }
        if (shouldShowReviewScreen != null) {
            getDocConfig().setShouldShowReviewScreen(shouldShowReviewScreen);
        }
        if (shouldShowInstructionPage != null) {
            getDocConfig().setShouldShowInstructionPage(shouldShowInstructionPage);
        }
        if (shouldEnableRetries != null) {
            getDocConfig().setShouldEnableRetries(shouldEnableRetries);
        }
        if (shouldDocAutoCapture != null) {
            getDocConfig().setShouldAutoCapture(shouldDocAutoCapture);
            if (shouldDocAutoCapture && autoCaptureDuration != null) {
                getDocConfig().setAutoCaptureDuration(autoCaptureDuration);
            }
        }
        if (docTypeValue != null) {
            try {
                HVDocConfig.Document doc = HVDocConfig.Document.valueOf(docTypeValue);
                if (aspectRatio != null) doc.setAspectRatio(this.aspectRatio);
                getDocConfig().setDocumentType(doc);
            } catch (Exception exp) {
                Log.e(TAG, exp.getMessage());
            }
        }
        if (shouldShowTrustLogos != null) {
            getDocConfig().setShowTrustLogos(shouldShowTrustLogos);
            System.out.println("shouldShowTrustLogos : " + shouldShowTrustLogos);
        }
        if (shouldShowCloseAlert != null) {
            getDocConfig().setShouldShowCloseAlert(shouldShowCloseAlert);
            if (shouldShowCloseAlert) {
                if (closeAlertDialogTitle != null && !closeAlertDialogTitle.isEmpty()) {
                    getDocConfig().setCloseAlertDialogTitle(closeAlertDialogTitle);
                }
                if (closeAlertDialogDescription != null && !closeAlertDialogDescription.isEmpty()) {
                    getDocConfig().setCloseAlertDialogDesc(closeAlertDialogDescription);
                }
            }
        }
        if (ocrApiDetails != null && !ocrApiDetails.isEmpty()) {
            setOCRAPIDetails(ocrApiDetails);
        }
        if (docCaptureUIStrings != null && !docCaptureUIStrings.isEmpty()) {
            setUIStrings(docCaptureUIStrings);
        }
    }


    @SuppressWarnings("unchecked")
    private void setOCRAPIDetails(Map<String, Object> ocrApiDetails) {

        String ocrEndpoint = (String) ocrApiDetails.get(FLHyperSnapConstants.OCR_ENDPOINT);
        String documentSideString = (String) ocrApiDetails.get(FLHyperSnapConstants.DOCUMENT_SIDE_STRING);
        Map<String, Object> params = (Map<String, Object>) ocrApiDetails.get(FLHyperSnapConstants.PARAMS);
        Map<String, Object> headers = (Map<String, Object>) ocrApiDetails.get(FLHyperSnapConstants.HEADERS);
        HVDocConfig.DocumentSide documentSide = HVDocConfig.DocumentSide.valueOf(documentSideString);

        JSONObject paramsObj = new JSONObject();
        JSONObject headersObj = new JSONObject();

        try {
            if (params != null) {
                paramsObj = FLHVNetworkHelper.convertMapToJson(params);
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
        }
        try {
            if (headers != null) {
                headersObj = FLHVNetworkHelper.convertMapToJson(headers);
            }
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
        }

        getDocConfig().setOCRDetails(ocrEndpoint,
                documentSide,
                paramsObj.toString(),
                headersObj.toString()
        );
    }


    private void setUIStrings(Map<String, Object> uiStringsMap) {
        //remove null ui strings
        uiStringsMap.values().removeAll(Collections.singleton(null));

        try {
            JSONObject uiStringsJsonObject = FLHVNetworkHelper.convertMapToJson(uiStringsMap);
            // Using customUIStrings
            getDocConfig().setCustomUIStrings(uiStringsJsonObject);
        } catch (JSONException e) {
            Log.e(TAG, e.getMessage());
        }
    }

    public void start(Context context, final DocCaptureCompletionHandler callback) {
        hasBeenCalled = false;
        HVDocsActivity.start(context, getDocConfig(), callback);
    }

    public List<Map<String, Object>> getMapsFromOnResult(HVError error, HVResponse hvResponse) {
        ArrayList<Map<String, Object>> arrayList = new ArrayList<>();
        Map<String, Object> errorObj = new HashMap<>();
        Map<String, Object> resultObj = new HashMap<>();

        try {
            if (error != null) {
                errorObj.put("errorCode", error.getErrorCode());
                errorObj.put("errorMessage", error.getErrorMessage());
                if (!hasBeenCalled) {
                    hasBeenCalled = true;
                }
            }
            if (hvResponse != null) {

                JSONObject result = hvResponse.getApiResult();
                JSONObject headers = hvResponse.getApiHeaders();

                String imageURI = hvResponse.getImageURI();
                String fullImageUri = hvResponse.getFullImageURI();
                String action = hvResponse.getAction();
                String retakeMessage = hvResponse.getRetakeMessage();
                String videoUri = hvResponse.getVideoUri();

                if (result != null) {
                    resultObj.put("apiResult", result.toString());
                    resultObj.put("imageUri", imageURI);
                    if (videoUri != null && !videoUri.isEmpty()) {
                        resultObj.put("videoUri", videoUri);
                    }
                    if (fullImageUri != null && !fullImageUri.isEmpty()) {
                        resultObj.put("fullImageUri", fullImageUri);
                    }
                    if (retakeMessage != null && !retakeMessage.isEmpty()) {
                        resultObj.put("retakeMessage", retakeMessage);
                    }
                    if (action != null && !action.isEmpty()) {
                        resultObj.put("action", action);
                    }
                }
                if (headers != null) {
                    resultObj.put("apiHeaders", headers.toString());
                }
                if (!hasBeenCalled) {
                    hasBeenCalled = true;
                }
            }

        } catch (Exception e) {
            Log.e(TAG, (e.getMessage()));

        }
        arrayList.add(errorObj);
        arrayList.add(resultObj);
        return arrayList;
    }
}
