package co.hyperverge.hypersnapsdk.hypersnapsdk_flutter;

import android.content.Context;
import android.util.Log;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import co.hyperverge.hvqrmodule.objects.HVQRConfig;
import co.hyperverge.hypersnapsdk.activities.HVQrScannerActivity;
import co.hyperverge.hypersnapsdk.listeners.QRScannerCompletionHandler;
import co.hyperverge.hypersnapsdk.objects.HVError;

public class FLHVQRScanCapture {

    private static final String TAG = FLHVQRScanCapture.class.getSimpleName();
    private HVQRConfig hvQRConfig;

    private boolean hasBeenCalled;

    public void createNewConfig() {
        this.hvQRConfig = new HVQRConfig();
    }

    public HVQRConfig getHVQRConfig() {
        if (this.hvQRConfig == null) {
            this.hvQRConfig = new HVQRConfig();
        }
        return this.hvQRConfig;
    }

    public void setHVQRConfig(Map<String, Object> hvQRConfig) {
        Boolean shouldShowInstructionPage = (Boolean) hvQRConfig.get(FLHyperSnapConstants.QR_CAPTURE_SHOULD_SHOW_INSTRUCTION_PAGE);

        if (shouldShowInstructionPage != null) {
            getHVQRConfig().setShowInstructions(shouldShowInstructionPage);
        }
    }

    public void start(Context context, final QRScannerCompletionHandler callback) {
        hasBeenCalled = false;
        HVQrScannerActivity.start(context, getHVQRConfig(), callback);
    }

    public List<Map<String, Object>> getALFromOnResultQRScan(HVError error, JSONObject result) {
        ArrayList<Map<String, Object>> arrayList = new ArrayList<>();

        Map<String, Object> errorObj = new HashMap<>();
        Map<String, Object> resultsObj = new HashMap<>();
        try {
            if (error != null) {
                errorObj.put("errorCode", error.getErrorCode());
                errorObj.put("errorMessage", error.getErrorMessage());
                if (!hasBeenCalled) {
                    hasBeenCalled = true;
                }
            }
            if (result != null) {
                Iterator<?> keys = result.keys();
                while (keys.hasNext()) {
                    String key = (String) keys.next();
                    try {
                        if (result.get(key) instanceof String) {
                            resultsObj.put(key, result.get(key));
                        } else if (result.get(key) instanceof JSONObject) {
                            resultsObj.put(key, result.get(key).toString());
                        }
                    } catch (Exception e) {
                        Log.e(TAG, Objects.requireNonNull(e.getMessage()));
                    }
                }
            }
            if (!hasBeenCalled) {
                hasBeenCalled = true;
            }

        } catch (Exception e) {
            Log.e(TAG, Objects.requireNonNull(e.getMessage()));
        }
        arrayList.add(errorObj);
        arrayList.add(resultsObj);
        return arrayList;
    }
}
