import 'dart:developer';

import 'package:flutter/services.dart';
import 'package:hypersnapsdk_flutter/hyper_snap_sdk_constants.dart';

import 'hv_result.dart';

/// HVBarcodeScanCapture - used to config and start barcode scan capture
class HVBarcodeScanCapture {
  static const MethodChannel _hvBarcodeScanCaptureChannel =
      MethodChannel(HyperSnapSDKConstants.hyperSnapChannel);

  /// start() is used to start barcode scan activity/view inside the client's application
  ///
  /// params :
  ///       hvBarcodeConfig(optional) : type - HVBarcodeConfig
  ///       onComplete : type - Function(HVResponse?,HVError?)
  ///
  /// returns : void
  static start(
      {HVBarcodeConfig? hvBarcodeConfig,
      required Function(HVResponse?, HVError?) onComplete}) async {
    Map resMap = {};
    HVError? hvError;
    HVResponse? hvResponse;

    try {
      resMap = await _hvBarcodeScanCaptureChannel
          .invokeMethod(HyperSnapSDKConstants.qrScanCaptureStart, {
        HyperSnapSDKConstants.hvQRConfig:
            hvBarcodeConfig?.toMap() ?? HVBarcodeConfig().toMap()
      });

      Map errorObj = resMap["errorObj"];
      Map resObj = resMap["resultObj"];
      hvError = (errorObj.isEmpty) ? null : HVError.fromMap(errorObj);
      hvResponse = (resObj.isEmpty) ? null : HVResponse.fromMap(resObj);
    } catch (e) {
      log(e.toString());
    }

    onComplete(hvResponse, hvError);
  }
}

/// HVBarcodeConfig - used to set configurations to Barcode Capture
class HVBarcodeConfig {
  bool? shouldShowInstructionPage;

  /// HVBarCodeConfig - used to set configurations to barcode Capture
  ///
  /// params :
  ///        shouldShowInstructionPage - type - bool , used to show instruction page before barcode capture
  HVBarcodeConfig({this.shouldShowInstructionPage});

  Map toMap() {
    return {
      HyperSnapSDKConstants.qrShouldShowInstructionPage:
          shouldShowInstructionPage
    };
  }
}
